//-- MainForm ------------------------------------------------------------------
//
//  ATTLFilter
//
//  Copyright  2006 Alexander Kamburov
//  wise_guybg@yahoo.com
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//------------------------------------------------------------------------------
{
  Code From ListAdapters Example
  Windows Packet Filter Kit 2.0
  Copyright(C) 2000-2003 NT Kernel Resources
  mailto: ndisrd@ntkernel.com
}
unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, TTLFilterUnit, StdCtrls, ComCtrls, XPMan, CoolTrayIcon, Menus,
  ImgList;

type
  TfrmMain = class(TForm)
    btnList: TButton;
    lvAdapters: TListView;
    XPManifest: TXPManifest;
    btnStart: TButton;
    lblAdapters: TLabel;
    btnStop: TButton;
    tiTray: TCoolTrayIcon;
    pmTray: TPopupMenu;
    miOpen: TMenuItem;
    miExit: TMenuItem;
    N2: TMenuItem;
    mmLog: TMemo;
    btnAbout: TButton;
    lblFilter: TLabel;
    btnSettings: TButton;
    ilMedium: TImageList;
    btnClear: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnListClick(Sender: TObject);
    procedure btnStartClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure tiTrayClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure miExitClick(Sender: TObject);
    procedure miOpenClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure btnSettingsClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
  private
    FExitConfirmation: Boolean;
    FQueryEndSession: Boolean;
    FTrace: Boolean;
    FFilteredAdapter: string;
    FBadTTL: Integer;
    FNewTTL: Integer;
    FTraceLogLen: Integer;
    FFilter: TTTLFilter;
    procedure LoadSettings;
    procedure SaveSettings;
    procedure WMQueryEndSession(var AMsg: TWMQueryEndSession); message WM_QUERYENDSESSION;
    procedure FilterStatus(const AMessage: string);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

const
  APP_REG_KEY = '\Software\Alexander Kamburov\ATTLFilter';

const
  NDIS_MEDIUM: array[0..12] of string = (
    'Ethernet (802.3)', 'Token Ring (802.5)', 'FDDI', 'WAN', 'LocalTalk', 'DIX',
    'ARCNET (raw)', 'ARCNET (878.2)', 'ATM', 'Wireless WAN', 'Infrared (IrDA)',
    'Connection-oriented WAN', 'IEEE 1394 (firewire) bus');

implementation

{$R *.dfm}

uses
  Registry, winpkf, AboutForm, SettingsForm;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  FQueryEndSession := false;
  frmMain.Caption := Application.Title;
  tiTray.Icon := Application.Icon;
  tiTray.Hint := Application.Title;
  LoadSettings;

  btnListClick(Sender);
  btnStartClick(Sender);
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  SaveSettings;
  btnStopClick(Sender);
end;

procedure TfrmMain.LoadSettings;
var
  Registry: TRegistry;
begin
  Registry := TRegistry.Create;
  try
    Registry.RootKey := HKEY_CURRENT_USER;
    if Registry.OpenKey(APP_REG_KEY, true) then
    begin
      if Registry.ValueExists('WindowLeft') then
        Left := Registry.ReadInteger('WindowLeft')
      else
        Left := 100;

      if Registry.ValueExists('WindowLeft') then
        Top := Registry.ReadInteger('WindowTop')
      else
        Top := 100;

      if Registry.ValueExists('BadTTL') then
        FBadTTL := Registry.ReadInteger('BadTTL')
      else
        FBadTTL := 1;

      if Registry.ValueExists('NewTTL') then
        FNewTTL := Registry.ReadInteger('NewTTL')
      else
        FNewTTL := 5;

      if Registry.ValueExists('Trace') then
        FTrace := Registry.ReadBool('Trace')
      else
        FTrace := false;

      if Registry.ValueExists('TraceLogLen') then
        FTraceLogLen := Registry.ReadInteger('TraceLogLen')
      else
        FTraceLogLen := 100;
        
      if Registry.ValueExists('FilteredAdapter') then
        FFilteredAdapter := Registry.ReadString('FilteredAdapter')
      else
        FFilteredAdapter := '';

      if Registry.ValueExists('ExitConfirmation') then
        FExitConfirmation := Registry.ReadBool('ExitConfirmation')
      else
        FExitConfirmation := true;

      if FFilteredAdapter = '' then
        WindowState := wsNormal
      else
        WindowState := wsMinimized;


      Registry.CloseKey;
    end;
  finally
    Registry.Free;
  end;
end;

procedure TfrmMain.SaveSettings;
var
  Registry: TRegistry;
begin
  Registry := TRegistry.Create;
  try
    Registry.RootKey := HKEY_CURRENT_USER;
    if Registry.OpenKey(APP_REG_KEY, true) then
    begin
      Registry.WriteInteger('WindowLeft', Left);
      Registry.WriteInteger('WindowTop', Top);
      Registry.WriteInteger('BadTTL', FBadTTL);
      Registry.WriteInteger('NewTTL', FNewTTL);
      Registry.WriteBool('Trace', FTrace);      
      Registry.WriteInteger('TraceLogLen', FTraceLogLen);      
      Registry.WriteString('FilteredAdapter', FFilteredAdapter);
      Registry.WriteBool('ExitConfirmation', FExitConfirmation);      
      Registry.CloseKey;
    end;
  finally
    Registry.Free;
  end;

end;

procedure TfrmMain.btnListClick(Sender: TObject);
var
  i: Integer;
  hFilt: THANDLE;
  Adapts: TCP_AdapterList;
  AdapterViewName, AdapterName: string;
  VerInfo: TOSVersionInfo;
  Item: TListItem;
  Medium: Cardinal;
begin
  lvAdapters.Items.BeginUpdate;
  try
    lvAdapters.Items.Clear;

    VerInfo.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
    GetVersionEx(VerInfo);

    // Create driver object
    hFilt := OpenFilterDriver('NDISRD');

    // Check if driver loaded
    if not IsDriverLoaded(hFilt) then
    begin
      MessageDlg('Helper driver failed to load or was not installed.',
        mtError, [mbOK], 0);
      Exit;
    end;

    try    
      GetTcpipBoundAdaptersInfo(hFilt, @Adapts);
      for i := 1 to Adapts.m_nAdapterCount do
      begin
        Item := lvAdapters.Items.Add;
        AdapterName := Adapts.m_szAdapterNameList[i];
        SetLength(AdapterName, StrLen(PAnsiChar(AdapterName)));

        // Convert internal network interface name to user-friendly one depending of the OS
        SetLength(AdapterViewName, 1024);
        if VerInfo.dwPlatformId = VER_PLATFORM_WIN32_NT then
            if VerInfo.dwMajorVersion = 4 then
              ConvertWindowsNTAdapterName(PAnsiChar(AdapterName),
                PAnsiChar(AdapterViewName), 1024)
            else
              ConvertWindows2000AdapterName(PAnsiChar(AdapterName),
                PAnsiChar(AdapterViewName), 1024)
        else
          ConvertWindows9xAdapterName(PAnsiChar(AdapterName),
            PAnsiChar(AdapterViewName), 1024);
        SetLength(AdapterViewName, StrLen(PAnsiChar(AdapterViewName)));
                              
        // Dump some network interface information
        Item.Caption := Format('(%d) %s', [i, AdapterViewName]);
        Item.Data := Pointer(i);
        Item.SubItems.Add(Format('%s', [AdapterName]));

        if FFilteredAdapter = AdapterName then
          Item.Selected := true;

        Item.SubItems.Add(Format('%.2x%.2x%.2x%.2x%.2x%.2x', [Adapts.m_czCurrentAddress[i][1], Adapts.m_czCurrentAddress[i][2], Adapts.m_czCurrentAddress[i][3], Adapts.m_czCurrentAddress[i][4], Adapts.m_czCurrentAddress[i][5], Adapts.m_czCurrentAddress[i][6]]));

        Medium := Adapts.m_nAdapterMediumList[i];

        if Medium <= High(NDIS_MEDIUM) then
        begin
          Item.SubItems.Add(NDIS_MEDIUM[Medium]);
          Item.ImageIndex := Medium;
        end
        else
        begin
          Item.SubItems.Add(Format('0x%.8X', [Medium]));
          Item.ImageIndex := Length(NDIS_MEDIUM);
        end;  

        Item.SubItems.Add(Format('%d', [Adapts.m_usMTU[i]]));
      end;
    finally
     CloseFilterDriver(hFilt);
    end;
  finally
    lvAdapters.Items.EndUpdate;
  end;
end;

procedure TfrmMain.btnStartClick(Sender: TObject);
begin
  if Assigned(lvAdapters.Selected) then
  begin
    if Assigned(FFilter) then
      btnStopClick(Sender);

    FFilter := TTTLFilter.Create(Cardinal(lvAdapters.Selected.Data),
      FBadTTL, FNewTTL, FilterStatus, FTrace);

    FFilteredAdapter := lvAdapters.Selected.SubItems[0];
    mmLog.Lines.Add(Format('Filtering %s...', [lvAdapters.Selected.Caption]));
  end
  else
  begin
    if Visible then
      MessageDlg('Please select a network adapter to filter.', mtInformation, [mbOK], 0);
  end;
end;

procedure TfrmMain.btnStopClick(Sender: TObject);
begin
  if Assigned(FFilter) then
  begin
    FFilter.Terminate;
    FFilter.WaitFor;
    FreeAndNil(FFilter);
    mmLog.Lines.Add('Filter closed.');    
  end;
  FFilteredAdapter := '';
end;

procedure TfrmMain.tiTrayClick(Sender: TObject);
begin
  if IsIconic(Application.Handle) then
    Application.Restore
  else
    Application.Minimize;
end;

procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := true;

  if not FQueryEndSession and FExitConfirmation then
    if MessageDlg('Do you really want to exit?', mtConfirmation,
       mbOKCancel, 0) = mrCancel then
      CanClose := false;
end;

procedure TfrmMain.WMQueryEndSession(var AMsg: TWMQueryEndSession);
begin
  FQueryEndSession := true;
  AMsg.Result := 1;
end;

procedure TfrmMain.miExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.miOpenClick(Sender: TObject);
begin
  tiTrayClick(Sender);
end;

procedure TfrmMain.btnAboutClick(Sender: TObject);
begin
  frmAbout.ShowModal;
end;

procedure TfrmMain.btnSettingsClick(Sender: TObject);
begin
  frmSettings.udBadTTL.Position := FBadTTL;
  frmSettings.udNewTTL.Position := FNewTTL;
  frmSettings.chbExitConfirmation.Checked := FExitConfirmation;

  if frmSettings.ShowModal <> mrOK then
    Exit;

  FBadTTL := frmSettings.udBadTTL.Position;
  FNewTTL := frmSettings.udNewTTL.Position;
  FExitConfirmation := frmSettings.chbExitConfirmation.Checked;  

  SaveSettings;
end;

procedure TfrmMain.FilterStatus(const AMessage: string);
begin
  while mmLog.Lines.Count >= FTraceLogLen do
    mmLog.Lines.Delete(0);

  mmLog.Lines.Add(AMessage);
end;

procedure TfrmMain.btnClearClick(Sender: TObject);
begin
  mmLog.Clear;
end;

end.
