//-- TTLFilterUnit -------------------------------------------------------------
//
//  ATTLFilter
//
//  Copyright  2006 Alexander Kamburov
//  wise_guybg@yahoo.com
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//------------------------------------------------------------------------------
{
  Code From PassThru Example
  Windows Packet Filter Kit 2.0
  Copyright(C) 2000-2003 NT Kernel Resources
  mailto: ndisrd@ntkernel.com

  Checksum by heilong
  Posted: Thu Jun 24, 2004 6:16 am
  URL: http://ntoskrnl.com/forum/viewtopic.php?t=114&sid=f2465c16fd4e662c5eded24ff17aefd8
}
unit TTLFilterUnit;

interface

uses
  Classes,
  SysUtils,
  Windows,
  StrUtils,
  Winsock,
  winpkf,
  iphlp;

type
  TOnFilterStatus = procedure(const AMessage: string) of object;
  TStatusType = (stError, stTrace);

  TTTLFilter = class(TThread)
  private
    FAdapterIndex: DWORD;
    FBadTTL: Byte;
    FNewTTL: Byte;
    FTracing: Boolean;
    FFilterStatus: string;
    FOnFilterStatus: TOnFilterStatus;
    procedure SetFilterStatus;
    procedure SetFilterStatusMessage(const AText: string;
      const AType: TStatusType; const ALevel: Integer = 0);
  protected
    procedure Execute; override;
  public  
    constructor Create(const AAdapterIndex: DWORD; const ABadTTL, ANewTTL: Byte;
      AOnFilterStatus: TOnFilterStatus; const ATracing: Boolean);
  end;

const
  NDIS_DRIVER_NAME = 'NDISRD';
  WAIT_FOR_EVENT = 1000;
  StatusType: array[TStatusType] of string = ('error', 'trace');

implementation

// Calculate IP checkcum without IP Options
function Checksum(p: PWord; Size: Word): Word;
var
  i: Integer;
  Sum: Cardinal;
begin

  Sum := 0;

  for i := 0 to Size - 1 do
  begin
    Sum := Sum + ntohs(p^);
    Inc(p);
  end;

  while (Sum shr 16) > 0 do
    Sum := (Sum and $FFFF) + (Sum shr 16);

  Result := not Sum;
end;

{ TTTLFilter }

constructor TTTLFilter.Create(const AAdapterIndex: DWORD;
  const ABadTTL, ANewTTL: Byte; AOnFilterStatus: TOnFilterStatus;
  const ATracing: Boolean);
begin
  inherited Create(true);
  FAdapterIndex := AAdapterIndex;
  FBadTTL := ABadTTL;
  FNewTTL := ANewTTL;
  FOnFilterStatus := AOnFilterStatus;
  FTracing := ATracing;
  FreeOnTerminate := false;
  Resume;
end;

procedure TTTLFilter.Execute;
var
  hFilt: THANDLE;
  Adapts: TCP_AdapterList;
  AdapterMode: ADAPTER_MODE;
  Buffer: INTERMEDIATE_BUFFER;
  ReadRequest: ETH_REQUEST;
  hEvent: THANDLE;
  hAdapter: THANDLE;
  pEtherHeader: TEtherHeaderPtr;
  pIPHeader: TIPHeaderPtr;
begin
  // Create driver object
  hFilt := OpenFilterDriver(NDIS_DRIVER_NAME);

  if not IsDriverLoaded(hFilt) then
  begin
    FFilterStatus := '[error] Driver not loaded.';
    Synchronize(SetFilterStatus);
    Exit;
  end;

  try
    // Get TCP/IP bound interfaces
    GetTcpipBoundAdaptersInfo(hFilt, @Adapts);

    // Check paramer values
    if FAdapterIndex > Adapts.m_nAdapterCount then
    begin
      SetFilterStatusMessage(
        'There is no network interface with such index on this system.',
        stError);
      Exit;
    end;

    // Create notification event
    hEvent := CreateEvent(nil, true, false, nil);
    if hEvent = 0 then
    begin
      SetFilterStatusMessage('Could not create event.', stError);
      Synchronize(SetFilterStatus);
      Exit;
    end;

    try
      hAdapter := Adapts.m_nAdapterHandle[FAdapterIndex];

      if SetPacketEvent(hFilt, hAdapter, hEvent) = 0 then
      begin
        SetFilterStatusMessage('Could not set receive event.', stError);
        Synchronize(SetFilterStatus);
        Exit;
      end;

      try
        // Initialize request
        ReadRequest.EthPacket.Buffer := @Buffer;
        ReadRequest.hAdapterHandle := hAdapter;

        AdapterMode.dwFlags := MSTCP_FLAG_RECV_TUNNEL;
        AdapterMode.hAdapterHandle := hAdapter;
        if SetAdapterMode(hFilt, @AdapterMode) = 0 then
        begin
          SetFilterStatusMessage('Could not set adapter mode.', stError);
          Synchronize(SetFilterStatus);
          Exit;
        end;
          
        try
          while not Terminated do
          begin
            // Wait for event
            WaitForSingleObject(hEvent, WAIT_FOR_EVENT);
            ResetEvent(hEvent);

            // Check the received request 
            while not Terminated and (ReadPacket(hFilt, @ReadRequest) <> 0) do
            begin
              try
                if FTracing then
                begin
                  SetFilterStatusMessage('Packet received.', stTrace, 0);
                  Synchronize(SetFilterStatus);
                end;      

                pEtherHeader := TEtherHeaderPtr(@Buffer.m_IBuffer);
                // Check for IP protocol and OnReceive flag
                if ntohs(pEtherHeader.h_proto) = ETH_P_IP then
                begin
                  if FTracing then
                  begin
                    SetFilterStatusMessage('Packet protocol: IP', stTrace, 1);
                    Synchronize(SetFilterStatus);
                  end;

                  pIPHeader := TIPHeaderPtr(Integer(pEtherHeader) +
                    SizeOf(TEtherHeader));

                  if FTracing then
                  begin
                    SetFilterStatusMessage(Format('TTL=%d', [pIPHeader.TTL]), stTrace, 1);
                    Synchronize(SetFilterStatus);
                  end;

                  // Check if TTL causes problems   
                  if pIPHeader.TTL <= FBadTTL then
                  begin
                    pIPHeader.TTL := FNewTTL;
                    pIPHeader.Checksum := 0;
                    pIPHeader.Checksum := htons(
                      Checksum(PWord(pIPHeader),
                      (SizeOf(TIPHeader) - SizeOf(DWORD)) div 2));

                    if FTracing then
                    begin
                      SetFilterStatusMessage('TTL changed.', stTrace, 2);
                      Synchronize(SetFilterStatus);
                    end;
                  end;
                end;
              finally
                // Indicate packet to MSTCP
                SendPacketToMstcp(hFilt, @ReadRequest);
              end;
            end;
          end;
        finally
          // Restore default mode
          AdapterMode.dwFlags := 0;
          AdapterMode.hAdapterHandle := hAdapter;
          SetAdapterMode(hFilt, @AdapterMode);
        end;
      finally
        // Set NULL event to release previously set event object
        SetPacketEvent(hFilt, hAdapter, 0);
      end;
    finally
      // Close Event
      if hEvent <> 0 then
        CloseHandle(hEvent);
    end;
  finally
    // Close driver object
    CloseFilterDriver(hFilt);
  end;
end;

procedure TTTLFilter.SetFilterStatusMessage(const AText: string;
  const AType: TStatusType; const ALevel: Integer);
begin
  FFilterStatus := Format('[%s] %s%s',
    [StatusType[AType], DupeString('  ', ALevel), AText])
end;

procedure TTTLFilter.SetFilterStatus;
begin
  if Assigned(FOnFilterStatus) then
    FOnFilterStatus(FFilterStatus);
end;

end.
