{                 Windows Packet Filter Kit 2.4 interface             }
{                 Copyright(C) 2002-2004 NT Kernel Resources          }
{                         mailto: ndisrd@ntkernel.com                 }
{                                                                     }
{                 Delphi import unit fixed on 2003-02-21 by           }
{                 Federico Simonetti (aka BackDream)                  }

unit winpkf;

interface

uses Windows;

const

// Some size constants
  ADAPTER_NAME_SIZE = 256;
  ADAPTER_LIST_SIZE = 32;
  ETHER_ADDR_LENGTH = 6;
  MAX_ETHER_FRAME = 1514;

// Adapter flags
  MSTCP_FLAG_SENT_TUNNEL = $1; // Receive packets sent by MSTCP
  MSTCP_FLAG_RECV_TUNNEL = $2; // Receive packets instead MSTCP
  MSTCP_FLAG_SENT_LISTEN = $4; // Receive packets sent by MSTCP, original ones delivered to the network
  MSTCP_FLAG_RECV_LISTEN = $8; // Receive packets received by MSTCP
  MSTCP_FLAG_FILTER_DIRECT = $10;	// In promiscuous mode TCP/IP stack receives all
												          // all packets in the ethernet segment, to prevent this set this flag
												          // All packets with destination MAC different from FF-FF-FF-FF-FF-FF and
												          // network interface current MAC will be blocked

// By default loopback packets are passed to original MSTCP handlers without processing,
// to change this behavior use the flags below
  MSTCP_FLAG_LOOPBACK_FILTER = $20;  // Pass loopback packet for processing
  MSTCP_FLAG_LOOPBACK_BLOCK	= $40;   // Silently drop loopback packets, this flag
												            // is recommended for usage in combination with
												            // promiscuous mode

// Device flags for intermediate buffer
  PACKET_FLAG_ON_SEND = 1;
  PACKET_FLAG_ON_RECEIVE = 2;

type

  PTCP_AdapterList = ^TCP_AdapterList;
  TCP_AdapterList = packed record
    m_nAdapterCount: DWORD; // Number of adapters
    m_szAdapterNameList: array[1..ADAPTER_LIST_SIZE, 1..ADAPTER_NAME_SIZE] of Char; // Array of adapter names
    m_nAdapterHandle: array[1..ADAPTER_LIST_SIZE] of THANDLE; // Array of adapter handles, this are key handles for any adapter relative operation
    m_nAdapterMediumList: array[1..ADAPTER_LIST_SIZE] of DWORD; // List of adapter mediums
    m_czCurrentAddress: array[1..ADAPTER_LIST_SIZE, 1..ETHER_ADDR_LENGTH] of Byte; // current (configured) ethernet address
    m_usMTU: array[1..ADAPTER_LIST_SIZE] of Word; // current adapter MTU
  end;

  PLIST_ENTRY = ^LIST_ENTRY;
  LIST_ENTRY = record
    Flink: PLIST_ENTRY;
    Blink: PLIST_ENTRY;
  end;

  PINTERMEDIATE_BUFFER = ^INTERMEDIATE_BUFFER;
  INTERMEDIATE_BUFFER = packed record
    m_qLink: LIST_ENTRY;
    m_dwDeviceFlags: DWORD;
    m_Length: DWORD;
    m_Flags: DWORD;
    m_IBuffer: array[1..MAX_ETHER_FRAME] of Byte;
  end;

  PNDISRD_ETH_Packet = ^NDISRD_ETH_Packet;
  NDISRD_ETH_Packet = packed record
    Buffer: PINTERMEDIATE_BUFFER;
  end;

  PETH_REQUEST = ^ETH_REQUEST;
  ETH_REQUEST = packed record
    hAdapterHandle: THANDLE;
    EthPacket: NDISRD_ETH_Packet;
  end;

  PADAPTER_MODE = ^ADAPTER_MODE;
  ADAPTER_MODE = packed record
    hAdapterHandle: THANDLE;
    dwFlags: DWORD;
  end;

  PADAPTER_EVENT = ^ADAPTER_EVENT;
  ADAPTER_EVENT = packed record
    hAdapterHandle: THANDLE;
    hEvent: THANDLE;
  end;

  PPACKET_OID_DATA = ^PACKET_OID_DATA;
  PACKET_OID_DATA = packed record
    hAdapterHandle: THANDLE;
    Oid: DWORD;
    Length: DWORD;
    Data: array[1..2] of Byte;
  end;

function OpenFilterDriver(pszFileName : Pchar): THANDLE; stdcall; external 'ndisapi.dll';
procedure CloseFilterDriver(hOpen: THANDLE); stdcall; external 'ndisapi.dll';
function GetDriverVersion(hOpen: THANDLE): DWORD; stdcall; external 'ndisapi.dll';
function GetTcpipBoundAdaptersInfo(hOpen: THANDLE; pAdapters: PTCP_AdapterList): DWORD; stdcall; external 'ndisapi.dll';
function SendPacketToMstcp(hOpen: THANDLE; pPacket: PETH_REQUEST): DWORD; stdcall; external 'ndisapi.dll';
function SendPacketToAdapter(hOpen: THANDLE; pPacket: PETH_REQUEST): DWORD; stdcall; external 'ndisapi.dll';
function ReadPacket(hOpen: THANDLE; pPacket: PETH_REQUEST): DWORD; stdcall; external 'ndisapi.dll';
function SetAdapterMode(hOpen: THANDLE; pMode: PADAPTER_MODE): DWORD; stdcall; external 'ndisapi.dll';
function GetAdapterMode(hOpen: THANDLE; pMode: PADAPTER_MODE): DWORD; stdcall; external 'ndisapi.dll';
function FlushAdapterPacketQueue(hOpen: THANDLE; hAdapter: THANDLE): DWORD; stdcall; external 'ndisapi.dll';
function GetAdapterPacketQueueSize(hOpen: THANDLE; hAdapter: THANDLE; pdwSize: PDWORD): DWORD; stdcall; external 'ndisapi.dll';
function SetPacketEvent(hOpen: THANDLE; hAdapter: THANDLE; hWin32Event: THANDLE): DWORD; stdcall; external 'ndisapi.dll';
function SetWANEvent(hOpen: THANDLE; hWin32Event: THANDLE): DWORD; stdcall; external 'ndisapi.dll';
function SetAdapterListChangeEvent(hOpen: THANDLE; hWin32Event: THANDLE): DWORD; stdcall; external 'ndisapi.dll';
function NdisrdRequest(hOpen: THANDLE; OidData: PPACKET_OID_DATA; dwSet: DWORD): DWORD; stdcall; external 'ndisapi.dll';
function SetHwPacketFilter(hOpen: THANDLE; hAdapter: THANDLE; dwFilter: DWORD):DWORD; stdcall; external 'ndisapi.dll';
function GetHwPacketFilter(hOpen: THANDLE; hAdapter: THANDLE; var dwFilter: DWORD):DWORD; stdcall; external 'ndisapi.dll';
function SetMTUDecrement(dwMTUDecrement: DWORD): DWORD; stdcall; external 'ndisapi.dll';
function GetMTUDecrement(): DWORD; stdcall; external 'ndisapi.dll';
function SetAdaptersStartupMode (dwStartupMode: DWORD): DWORD; stdcall; external 'ndisapi.dll';
function GetAdaptersStartupMode(): DWORD; stdcall; external 'ndisapi.dll';
function IsDriverLoaded(hOpen: THANDLE): Boolean; stdcall; external 'ndisapi.dll';
function GetBytesReturned(hOpen: THANDLE): DWORD; stdcall; external 'ndisapi.dll';
function ConvertWindowsNTAdapterName(szAdapterName: Pchar; szUserFriendlyName: Pchar; dwUserFriendlyNameLength: DWORD): DWORD; stdcall; external 'ndisapi.dll';
function ConvertWindows2000AdapterName(szAdapterName: Pchar; szUserFriendlyName: Pchar; dwUserFriendlyNameLength: DWORD): DWORD; stdcall; external 'ndisapi.dll';
function ConvertWindows9xAdapterName(szAdapterName: Pchar; szUserFriendlyName: Pchar; dwUserFriendlyNameLength: DWORD): DWORD; stdcall; external 'ndisapi.dll';

implementation

end.
